#!/usr/bin/env python

from esm_parser.yaml_to_dict import yaml_file_to_dict
import yaml

def main():
    s2m = yaml_file_to_dict("../configs/esm_master/setups2models.yaml")
    with open("../configs/esm_master/defaults.yaml", "w") as yml:
        yaml.dump(s2m["defaults"], yml)
        del s2m["defaults"]
    with open("../configs/esm_master/esm-software.yaml", "w") as yml:
        yaml.dump(s2m["esm-software"], yml)
        del s2m["esm-software"]
    for thing in ["setups", "components", "couplings"]:
        for component in s2m[thing]:
            with open("../configs/esm_master/"+thing+"/"+component+".yaml", "w") as yml:
                yaml.dump(s2m[thing][component], yml)



if __name__ == "__main__":
    main()



#!/usr/bin/env python

# TODO: add a documentation

import re
import shutil
import os
import pathlib

# directory and file paths
conf_dir = pathlib.Path.cwd() / 'config'
mh_fname = conf_dir / "mh-linux"
backup_fname = conf_dir / "mh-linux.bak"

# convert them to strings since versions prior to 3.6 os module does not like
# pathlib objects
mh_fname = str(mh_fname)
backup_fname = str(backup_fname)

# create a backup file if it does not exit. Copy the contents of the original
# file to the backup file
if not os.path.isfile(backup_fname):
    shutil.copyfile(mh_fname, backup_fname)

# backup file is read-only and original will be overwritten
backup_file = open(backup_fname, 'r')

# erase the contents of the original file
mh_file = open(mh_fname, 'r+')
mh_file.truncate(0)

# read the backup file line by line and copy the contents to mh file
line = backup_file.readline()
line_num = 1
inside_gcc_block = False         
is_file_changed = False 

# regex search patterns
gcc_block_pattern = r'default\|gcc*.\)'
fc_pattern = r'^\s*FC\s*=\s*[a-zA-Z0-9_]*$'
f77_pattern = r'^\s*F77\s*=\s*[a-zA-Z0-9_]*$'


while line:    
    # check for the start of the GCC case statement
    if re.search(gcc_block_pattern, line) != None:
        inside_gcc_block = True
        #print(f"found the gcc block on line {line_num}")
    
    # inside the GCC section change
    #     FC  = gfortran   -->   FC  = mpif90
    #     F77 = gfortran   -->   F77 = mpif90
    if inside_gcc_block:
        match_FC = re.search(fc_pattern, line)
        match_F77 = re.search(f77_pattern, line)
        
        if match_FC != None:            
            #print(f"FC found on line {line_num}")
            num_spaces = line.find('FC')
            line = ' ' * num_spaces + 'FC = mpif90\n' 
            is_file_changed = True
        
        elif match_F77 != None:
            #print(f"F77 found on line {line_num}")
            num_spaces = line.find('F77')
            line = ' ' * num_spaces + 'F77 = mpif90\n' 
            is_file_changed = True
    
    # end of the GCC case statement
    if ';;' in line and inside_gcc_block:
        #print(f"Block end on line {line_num}")
        inside_gcc_block = False
        
    # write the lines to the mh file
    mh_file.write(line)    
    line = backup_file.readline()
    line_num += 1

# close the files
backup_file.close()            
mh_file.close()            

if is_file_changed:
    print(':: change_mh_file: compiler blocks in the mh-file have been updated')



#!/usr/bin/env python3

import argparse
import yaml

def merge(a, b, path=None):
    "merges b into a"
    if path is None: path = []
    for key in b:
        if key in a:
            if isinstance(a[key], dict) and isinstance(b[key], dict):
                merge(a[key], b[key], path + [str(key)])
            elif a[key] == b[key]:
                pass # same leaf value
            else:
                if isinstance(a[key], list) and isinstance(b[key], list):
                    a[key] = list(set(a[key] + b[key]))
                else:
                    print('Conflict at %s' % '.'.join(path + [str(key)]))
                    print(f"a: {a[key]}")
                    print(f"b: {b[key]}")
                    a_or_b = input("Please choose a or b: ")
                    print(f"Ok, setting: {a_or_b}[{key}]")
                    d_to_use = locals()[a_or_b]
                    a[key] = d_to_use[key]
                    #raise Exception('Conflict at %s' % '.'.join(path + [str(key)]))
        else:
            a[key] = b[key]
    return a

def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("yaml1")
    parser.add_argument("yaml2")
    parser.add_argument("yaml_out")
    return parser.parse_args()

def yaml_file_to_dict(filepath):
    """
    Given a yaml file, returns a corresponding dictionary.

    If you do not give an extension, tries again after appending one.

    Parameters
    ----------
    filepath : str
        Where to get the YAML file from

    Returns
    -------
    dict
        A dictionary representation of the yaml file.
    """
    with open(filepath) as yaml_file:
        return yaml.load(yaml_file, Loader=yaml.FullLoader)


class quoted(str):
    pass

def quoted_presenter(dumper, data):
    return dumper.represent_scalar('tag:yaml.org,2002:str', data, style='"')
yaml.add_representer(quoted, quoted_presenter)

class literal(str):
    pass

def literal_presenter(dumper, data):
    return dumper.represent_scalar('tag:yaml.org,2002:str', data, style='|')
yaml.add_representer(literal, literal_presenter)

def ordered_dict_presenter(dumper, data):
    return dumper.represent_dict(data.items())


def transform_strs(d, path=None):
    if path is None: path = []
    for k in d:
        if isinstance(d[k], dict):
            transform_strs(d[k], path + [str(k)])
        elif isinstance(d[k], list):
            new_list = []
            for item in d[k]:
                if isinstance(item, str):
                    new_item = literal(item) if "\n" in item or ";" in item else quoted(item)
                else:
                    new_item = item
                new_list.append(item)
            d[k] = new_list
        elif isinstance(d[k], str):
            item = d[k]
            d[k] =  literal(item) if "\n" in item or ";" in item else quoted(item)
        elif isinstance(d[k], int):
            continue
        else:
            print(d[k], type(d[k]))
            raise Exception("Do not understand...")
    return d

def main():
    args = parse_args()
    d1 = yaml_file_to_dict(args.yaml1)
    d2 = yaml_file_to_dict(args.yaml2)
    d1 = merge(d1, d2)

    with open(args.yaml_out, "w") as yaml_out:
        yaml.dump(transform_strs(d1), yaml_out)

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
from esm_parser.yaml_to_dict import yaml_file_to_dict
import yaml

import os
import sys


def main():
    workflow = yaml_file_to_dict("../.github/workflows/CI-Ollie.yml.example")
    workflow["on"] = workflow[True]
    del workflow[True]

    for setup_yaml in os.listdir("../configs/esm_master/setups/"):
        setup_name = setup_yaml.replace(".yaml", "")
        setup = yaml_file_to_dict("../configs/esm_master/setups/" + setup_name)
        for version in setup.get("available_versions", []):
            workflow["name"] = "CI-Ollie-install-" + setup_name + "-" + version
            for event_type in ["push", "pull_request"]:
                all_components = []
                for coupling in setup["choose_version"][version]["couplings"]:
                    components = coupling.split("+")
                    components = [item.split("-")[0] for item in components]
                    all_components += components
                components = list(set(all_components))
                workflow["on"][event_type]["branches"] = [
                    "release",
                    "develop",
                    "*prep-release*",
                ]
                workflow["on"][event_type]["paths"] = (
                    ["configs/esm_master/setups/" + setup_name + ".yaml"]
                    + [
                        "configs/esm_master/components/" + component + ".yaml"
                        for component in components
                    ]
                    + [
                        "configs/esm_master/couplings/" + coupling + ".yaml"
                        for coupling in setup["choose_version"][version]["couplings"]
                    ]
                )
            workflow["jobs"]["install_models"]["strategy"]["matrix"]["model"] = [
                setup_name + "-" + version
            ]
            with open(
                "../.github/workflows/"
                + "CI-Ollie-install-"
                + setup_name
                + "-"
                + version
                + ".yml",
                "w",
            ) as yml:
                yaml.dump(workflow, yml)


if __name__ == "__main__":
    main()



"""
=================================
ESM Tools (Package Documentation)
=================================

This package contains almost no code, but instead is where most of the YAML
configurations files are distributed. Default namelists are included as well as
example runscripts. This section **only** documents the code contained in the
module, please refer to the handbook for user documentation as well as API
documentation for the various sub-modules of the project.

Accessing Configuration
-----------------------

To access a particular configuration, you can use::

    >>> ollie_config = read_config_file("machines/ollie")

Important note here is that the configuration file **has not yet been parsed**,
so it's just the dictionary representation of the YAML.
"""

__author__ = """Dirk Barbi, Paul Gierz"""
__email__ = "dirk.barbi@awi.de"
__version__ = "5.1.24"

import os
import shutil
import sys

import pkg_resources
import yaml


def _get_namelist_filepath_standard_install(namelist):
    return pkg_resources.resource_filename("esm_tools.namelists", namelist)

def _get_namelist_filepath_editable_install(namelist):
    return pkg_resources.resource_filename("namelists", namelist)

def _get_config_filepath_standard_install(config):
    return pkg_resources.resource_filename("esm_tools.configs", config)

def _get_config_filepath_editable_install(config):
    return pkg_resources.resource_filename("configs", config)

def _get_runscript_filepath_standard_install(runscript):
    return pkg_resources.resource_filename("esm_tools.runscripts", runscript)

def _get_runscript_filepath_editable_install(runscript):
    return pkg_resources.resource_filename("runscripts", runscript)


def get_config_as_str(config):
    return pkg_resources.resource_string("esm_tools.configs", config)

def _list_config_dir_standard_install(dir_path):
    return pkg_resources.resource_listdir("esm_tools.configs", dir_path)

def _list_config_dir_editable_install(dir_path):
    return pkg_resources.resource_listdir("configs", dir_path)

# For more information on how this works, see here:
# https://stackoverflow.com/questions/62550952/including-package-data-python-from-top-level-when-package-is-in-subdirectory/62552188#62552188
def _read_config_standard_install(config):
    """
    Reads a config file for the case that you have done a standard pip install.

    Parameters
    ----------
    config : str
        The configuration to read, e.g. "machines/ollie.yaml"

    Returns
    -------
    dict :
        A dictionary representation of the configuration
    """
    if not config.endswith(".yaml"):
        config += ".yaml"
    configstr = pkg_resources.resource_string("esm_tools.configs", config)
    # configstr = pkg_resources.resource_string("configs", config)
    configdict = yaml.load(configstr, Loader=yaml.FullLoader)
    return configdict

def _read_namelist_standard_install(nml):
    """
    Reads a namelist file for the case that you have done a standard pip install.

    Parameters
    ----------
    nml : str
        The nml to read, e.g. "echam/6.3.04p2/PALEO/namelist.echam"

    Returns
    -------
    str :
        A string representation of the namelist file. This should later be
        passed to the f90nml package.
    """
    return pkg_resources.resource_string("esm_tools.namelists", nml)


def _read_config_editable_install(config):
    """
    Reads a config file for the case you have done an editable/develop install
    (e.g. pip -e).

    Parameters
    ----------
    dict :
        A dictionary representation of the configuration
    """
    if not config.endswith(".yaml"):
        config += ".yaml"
    # Note the only difference here is apparently swapping out
    # esm_tools.configs for just configs. Not sure how that works, but it seems
    # to be fine...
    configstr = pkg_resources.resource_string("configs", config)
    configdict = yaml.load(configstr, Loader=yaml.FullLoader)
    return configdict


def _copy_config_folder_standard_install(dest_path):
    src_path = pkg_resources.resource_filename("esm_tools.configs", ".")
    return shutil.copytree(src_path, dest_path)

def _copy_config_folder_editable_install(dest_path):
    src_path = pkg_resources.resource_filename("configs", ".")
    return shutil.copytree(src_path, dest_path)

def _copy_namelist_folder_standard_install(dest_path):
    src_path = pkg_resources.resource_filename("esm_tools.namelists", ".")
    return shutil.copytree(src_path, dest_path)

def _copy_namelist_folder_editable_install(dest_path):
    src_path = pkg_resources.resource_filename("namelists", ".")
    return shutil.copytree(src_path, dest_path)

def _copy_runscript_folder_standard_install(dest_path):
    src_path = pkg_resources.resource_filename("esm_tools.runscripts", ".")
    return shutil.copytree(src_path, dest_path)

def _copy_runscript_folder_editable_install(dest_path):
    src_path = pkg_resources.resource_filename("runscripts", ".")
    return shutil.copytree(src_path, dest_path)


def _read_namelist_editable_install(nml):
    """
    Reads a namelist file for the case that you have done an editable/develop install.

    Parameters
    ----------
    nml : str
        The nml to read, e.g. "echam/6.3.04p2/PALEO/namelist.echam"

    Returns
    -------
    str :
        A string representation of the namelist file. This should later be
        passed to the f90nml package.
    """
    return pkg_resources.resource_string("namelists", nml)

# PG: Blatant theft:
# https://stackoverflow.com/questions/42582801/check-whether-a-python-package-has-been-installed-in-editable-egg-link-mode
def _dist_is_editable(dist):
    """Is distribution an editable install?"""
    for path_item in sys.path:
        egg_link = os.path.join(path_item, dist.replace("_", "-") + ".egg-link")
        if os.path.isfile(egg_link):
            return True
    return False


EDITABLE_INSTALL = _dist_is_editable("esm_tools")
"""
bool : Shows if the installation is installed in editable mode or not.
"""


# Create a function that "does the right thing", depending on how this module
# is installed
def read_config_file(config):
    """
    Reads a configuration file, which should be seperated by "/". For example,
    "machines/ollie" will retrieve the (unparsed) configuration of the Ollie
    supercomputer.

    Parameters
    ----------
    config : str
        Configuration to get, e.g. machines/ollie.yaml, or echam/echam. You may
        omit the ".yaml" ending if you want, it will be appended automatically
        if not already there.

    Returns
    -------
    dict :
        A dictionary representation of the config.
    """
    if EDITABLE_INSTALL:
        return _read_config_editable_install(config)
    return _read_config_standard_install(config)


def list_config_dir(dirpath):
    if EDITABLE_INSTALL:
        return _list_config_dir_editable_install(dirpath)
    return _list_config_dir_standard_install(dirpath)

def copy_config_folder(dest_path):
    if EDITABLE_INSTALL:
        return _copy_config_folder_editable_install(dest_path)
    return _copy_config_folder_standard_install(dest_path)

def copy_namelist_folder(dest_path):
    if EDITABLE_INSTALL:
        return _copy_namelist_folder_editable_install(dest_path)
    return _copy_namelist_folder_standard_install(dest_path)

def copy_runscript_folder(dest_path):
    if EDITABLE_INSTALL:
        return _copy_runscript_folder_editable_install(dest_path)
    return _copy_runscript_folder_standard_install(dest_path)

def get_namelist_filepath(namelist):
    if EDITABLE_INSTALL:
        return _get_namelist_filepath_editable_install(namelist)
    return _get_namelist_filepath_standard_install(namelist)

def get_config_filepath(config):
    if EDITABLE_INSTALL:
        return _get_config_filepath_editable_install(config)
    return _get_config_filepath_standard_install(config)

def get_runscript_filepath(runscript):
    if EDITABLE_INSTALL:
        return _get_runscript_filepath_editable_install(runscript)
    return _get_runscript_filepath_standard_install(runscript)


def read_namelist_file(nml):
    """Reads a namelist file from a path, seperated by "/". Similar to ``read_config_file``

    Parameters
    ----------
    nml : str
        The namelist to load

    Returns
    -------
    str :
        A string of the namelist file
    """
    if EDITABLE_INSTALL:
        return _read_namelist_editable_install(nml)
    return _read_namelist_standard_install(nml)



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages
from os import getenv

with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("stuff/HISTORY.rst") as history_file:
    history = history_file.read()

requirements = [
    "pyyaml>=5.1",
    "esm_master @ git+https://github.com/esm-tools/esm_master.git",
    "esm_runscripts @ git+https://github.com/esm-tools/esm_runscripts.git",
    "esm_rcfile @ git+https://github.com/esm-tools/esm_rcfile.git",
    "esm_version_checker @ git+https://github.com/esm-tools/esm_version_checker.git",
    "gfw-creator",
]

setup_requirements = []

test_requirements = []

thisfolder = getenv("PWD")  # that somehow works, even though pip copies everything to
# a temp folder... seemingly without changing the PWD var
setup(
    author="Dirk Barbi",
    author_email="dirk.barbi@awi.de",
    python_requires=">=3.5",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    description="ESM Tools external infrastructure for Earth System Modelling",
    install_requires=requirements,
    license="GNU General Public License v2",
    long_description=readme + "\n\n" + history,
    include_package_data=True,
    keywords="esm_tools",
    name="esm-tools",
    packages=["esm_tools", "esm_tools.configs", "esm_tools.namelists", "esm_tools.runscripts"],
    package_dir={'esm_tools.configs': "configs", "esm_tools.namelists": "namelists", "esm_tools.runscripts": "runscripts"},
    package_data={'esm_tools.configs': ['../configs/*'], "esm_tools.namelists": ["../namelists/*"], "esm_tools.runscripts": ["../runscripts/*"]},
    setup_requires=setup_requirements,
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/esm_tools/esm_tools",
    version="5.1.24",
    zip_safe=False,
)

